;
; Voltmeter.asm
;
; Created: 10. 12. 2020 10:18:47
; Author : Vladimir Visnovsk
;

; .Include "tn24def.inc"

; IO pripojenia
; AD0 - Referencia
; AD1 - Zaporny
; AD2 - Kladny
; PA3 - Minus
; PA4 - CLK
; PA5 - DAT
; PA6 - 
; PA7 - STB
; PB2 - EXT

; Konstanty
 .def	T		= R16
 .def	A		= R17
 .def	Dig1	= R18
 .def	Dig10	= R19
 .def	Dig100	= R20
 .def	Cnt		= R21
 .def	Ref		= R3
 .equ	CLK		= 4
 .equ	DAT		= 5
 .equ	STB		= 7	
 .equ	Minus	= 3	

App_Init:
; Inicializacia

 ; Nastavenie stacku
 ldi	T		, Low  (RAMEND)
 out	SPL		, T

 ; Nastavenie portu A
 ldi	T		, 0b10111000
 out	DDRA	, T
 ldi	T		, 0b00000000
 out	PortA	, T
 ; Nastavenie portu B
 ldi	T		, 0b00000000
 out	DDRB	, T
 ldi	T		, 0b11111111
 out	PortB	, T

; Nastavenie AD prevodnika
 ; external ref	- 0b01
 ; internal 5V	- 0b00 
 .Equ   Ref_ext	= 0b01000000
 ; zapnutie AD
 ; ADEN
 sbi	ADCSRA	, ADEN

 ; Nastavenie referenci pre AD podla vstupy PB2
 ldi	T		, Ref_ext			; referencia externa
 ; Zistenie vtupu Ext - nastavenie referencie
 sbic	PortB	, 2					; Zistenie stavu na externom stupe
  clr	T
 mov	Ref		, T					; ulozenie referencie

; Slucka - hlavny program
App_Loop:

 ; Vynulovanie premennych pre vnutornu slucku
 clr	XL
 clr	XH
 ldi	Cnt		, 8

 ; Vnutorna slucka
 App_Input_Loop:

 ; Nacitanie AD - negativny
 ; ADC1 vstup	- 000001 
 ldi	T		, 0b01
 add	T		, Ref
 out	ADMUX	, T
 rcall	Delay
 ; spustenie konverzie ADSC
 sbi	ADCSRA	, ADSC
 ; Cakanie kym neskonci konverzia
 ADC_Negative_Loop:
  sbic	ADCSRA	, ADSC
   rjmp ADC_Negative_Loop
 ; nacitanie dat
 in		ZL		, ADCL
 in		ZH		, ADCH

 ; zistenie ci je namerana hodnota 0
 adiw	ZL		, 0
  breq App_Load_Positiv
  ; zaporna hodnota
   sbi	PortA	, Minus
   rjmp App_Send

 App_Load_Positiv:
 ; Nacitanie AD - pozitivny
 ; ADC2 vstup	- 000010 
 ldi	T		, 0b10
 add	T		, Ref
 out	ADMUX	, T
 rcall	Delay
; spustenie konverzie ADSC
 sbi	ADCSRA	, ADSC
 ; Cakanie kym neskonci konverzia
 ADC_Positive_Loop:
  sbic	ADCSRA	, ADSC
   rjmp ADC_Positive_Loop
 ; nacitanie dat
 in		ZL		, ADCL
 in		ZH		, ADCH

 ; Zmazanie znamienka minus
 cbi	PortA	, Minus 

 App_Send:
 ; Pripocitanie hodnot do statistiky
 add	XL		, ZL
 adc	XH		, ZH

 ; Pocitadlo sluciek
 dec	Cnt
  brne	App_Input_Loop

; Delenie 4
 lsr	XH
 ror	XL
 lsr	XH
 ror	XL
 lsr	XH
 ror	XL

 ; odoslanie nacitanych hodnot
 rcall SendNumber 

 ; Cakaj
 rcall	Delay
 rcall	Delay
 rcall	Delay
 
 ; Koniec slucky   
 rjmp App_Loop


; Odoslanie celeho cisla - X
SendNumber:
 ; Ak je cislo vecsie ako 1000 vypis ---
 ldi	T		, Low (1000)
 cp		XL		, T
 ldi	T		, High(1000)
 cpc	XH		, T
  brlo SendNumber_Convert
   ; Nadlimit
   ldi	A		, 0b01000000
   rcall SendData 
   ldi	A		, 0b01000000
   rcall SendData 
   ldi	A		, 0b01000000
   rcall SendData
   rcall Done
   ; Koniec
   ret 
 ; Konverzia 
 SendNumber_Convert:  
 ; Delenie cisla X / 100
 ldi	YL		, Low (100)
 ldi	YH		, High(100)
 rcall Div
 mov	Dig100	, ZL
 ; Delenie cisla X / 10
 ldi	YL		, Low (10)
 ldi	YH		, High(10)
 rcall Div
 mov	Dig10	, ZL
 ; Zbytok su jednotky
 ; XL zvysko po deleneni
 mov	Dig1	, XL
 ; Odoslane znakov
 mov	A		, Dig1
 rcall SendDigit
 set
 mov	A		, Dig10
 rcall SendDigit
 mov	A		, Dig100
 cpi	A		, 0			; odstranenie nuly
  brne SendNumber_NoNul
   ldi	A		, 255
  SendNumber_NoNul:
 rcall SendDigit
 rcall Done
 ; Koniec
ret

; Odoslanie cisla
SendDigit:
 ; Rozlisenie znaku
 cpi	A		, 16
  brlo SendDigit_Number
   ; Znak mimo rozsah - prazny znak
   clr	A		
   rjmp SendDigit_Disp
 ; Hladanie znaku
 SendDigit_Number:
 ; Nacitanie adresy tabulky
 ldi	ZL		, Low (Table << 1)
 ldi	ZH		, High(Table << 1)
 ; Pripocitanie indexu znaku
 clr	T
 add	ZL		, A
 adc	ZH		, T
 ; Nacitanie znaku
 lpm	A		, Z
 ; Dodanie ciarky
 bld	A		, 7
 ; Odoslanie
 SendDigit_Disp:
 rcall	SendData
 ; Koniec
ret

; Pocedura odoslania bytu
SendData:
 ; Nacitanie portu
 in		T		, PortA
 ; Odosielanie bitu
 bst	A		, 7		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 6		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 5 	; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 4		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 3		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 2		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 1		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Odosielanie bitu
 bst	A		, 0		; Nacitany bit z A
 bld	T		, DAT
 out	PortA	, T
 nop
 ; Impulz - clk
 sbi	PortA	, CLK
 nop
 cbi	PortA	, CLK
 ; Koniec
 clt
Ret

; Generovane impulzu STB - hotovo
Done:
 sbi	PortA	, STB
 nop
 cbi	PortA	, STB
 ; Koniec
Ret

; Delene, kolko krat sa nachadza X/Y=Z
Div:
 ; Vymazat pocitadlo
 clr	ZL
 clr	ZH
 Div_Loop:
 ; Porovnanie pre ukoncenie
 cp		XL		, YL
 cpc	XH		, YH  
  brlo Div_End
   ; Pripocitanie ku Z
   adiw	ZL		, 1 
   ; Odpocet od X cislo Y
   sub	XL		, YL
   sbc	XH		, YH
   ; Slucka
   rjmp Div_Loop
 ; Koniec       
 Div_End:
Ret

; Delay
 Delay:
  ; Nastavenie casu
  ldi	ZH		, 0xF0
  ldi	ZL		, 0x00
  ; Slucka
  Delay_Loop:
   Nop
   Nop
   Nop
   Nop
   Nop
   Subi	ZL		, 1
    Brne Delay_Loop
 ; Koniec
 Ret

; Tabulka znakov
Table: 
 .db 0b00111111, 0b00000110, 0b01011011, 0b01001111, 0b01100110, 0b01101101, 0b01111101, 0b00000111
 .db 0b01111111, 0b01101111, 0b01110111, 0b01111100, 0b00111001, 0b01011110, 0b01111001, 0b01110001